@@ -67,6 +67,7 @@ typedef enum
  * @GST_VAAPI_PICTURE_FLAG_TFF: top-field-first
  * @GST_VAAPI_PICTURE_FLAG_ONEFIELD: only one field is valid
  * @GST_VAAPI_PICTURE_FLAG_MVC: multiview component
+ * @GST_VAAPI_PICTURE_FLAG_RFF: repeat-first-field
  * @GST_VAAPI_PICTURE_FLAG_LAST: first flag that can be used by subclasses
  *
  * Enum values used for #GstVaapiPicture flags.
@@ -81,7 +82,8 @@ typedef enum
   GST_VAAPI_PICTURE_FLAG_TFF        = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 5),
   GST_VAAPI_PICTURE_FLAG_ONEFIELD   = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 6),
   GST_VAAPI_PICTURE_FLAG_MVC        = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 7),
-  GST_VAAPI_PICTURE_FLAG_LAST       = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 8),
+  GST_VAAPI_PICTURE_FLAG_RFF        = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 8),
+  GST_VAAPI_PICTURE_FLAG_LAST       = (GST_VAAPI_CODEC_OBJECT_FLAG_LAST << 9),
 } GstVaapiPictureFlags;
 
 #define GST_VAAPI_PICTURE_FLAGS         GST_VAAPI_MINI_OBJECT_FLAGS
@@ -107,6 +109,9 @@ typedef enum
 #define GST_VAAPI_PICTURE_IS_TFF(picture) \
   GST_VAAPI_PICTURE_FLAG_IS_SET (picture, GST_VAAPI_PICTURE_FLAG_TFF)
 
+#define GST_VAAPI_PICTURE_IS_RFF(picture) \
+  GST_VAAPI_PICTURE_FLAG_IS_SET (picture, GST_VAAPI_PICTURE_FLAG_RFF)
+
 #define GST_VAAPI_PICTURE_IS_ONEFIELD(picture) \
   GST_VAAPI_PICTURE_FLAG_IS_SET (picture, GST_VAAPI_PICTURE_FLAG_ONEFIELD)
 
