@@ -862,10 +862,11 @@ gst_vaapi_display_destroy (GstVaapiDisplay * display)
 
   gst_vaapi_display_replace_internal (&priv->parent, NULL);
 
-  g_mutex_lock (&g_display_cache_lock);
-  if (priv->cache)
+  if (priv->cache) {
+    gst_vaapi_display_cache_lock (priv->cache);
     gst_vaapi_display_cache_remove (priv->cache, display);
-  g_mutex_unlock (&g_display_cache_lock);
+    gst_vaapi_display_cache_unlock (priv->cache);
+  }
   gst_vaapi_display_cache_replace (&priv->cache, NULL);
   free_display_cache ();
 }
@@ -905,6 +906,7 @@ gst_vaapi_display_create_unlocked (GstVaapiDisplay * display,
         return FALSE;
       priv->display = info.va_display;
       priv->display_type = info.display_type;
+      priv->native_display = info.native_display;
       if (klass->get_size)
         klass->get_size (display, &priv->width, &priv->height);
       if (klass->get_size_mm)
@@ -953,9 +955,9 @@ gst_vaapi_display_create (GstVaapiDisplay * display,
   gst_vaapi_display_cache_replace (&priv->cache, cache);
   gst_vaapi_display_cache_unref (cache);
 
-  g_mutex_lock (&g_display_cache_lock);
+  gst_vaapi_display_cache_lock (cache);
   success = gst_vaapi_display_create_unlocked (display, init_type, init_value);
-  g_mutex_unlock (&g_display_cache_lock);
+  gst_vaapi_display_cache_unlock (cache);
   return success;
 }
 
@@ -1279,10 +1281,29 @@ gst_vaapi_display_flush (GstVaapiDisplay * display)
 }
 
 /**
- * gst_vaapi_display_get_display:
+ * gst_vaapi_display_get_class_type:
  * @display: a #GstVaapiDisplay
  *
- * Returns the #GstVaapiDisplayType bound to @display.
+ * Returns the #GstVaapiDisplayType of @display. This is the type of
+ * the object, thus the associated class, not the type of the VA
+ * display.
+ *
+ * Return value: the #GstVaapiDisplayType
+ */
+GstVaapiDisplayType
+gst_vaapi_display_get_class_type (GstVaapiDisplay * display)
+{
+  g_return_val_if_fail (display != NULL, GST_VAAPI_DISPLAY_TYPE_ANY);
+
+  return GST_VAAPI_DISPLAY_GET_CLASS_TYPE (display);
+}
+
+/**
+ * gst_vaapi_display_get_display_type:
+ * @display: a #GstVaapiDisplay
+ *
+ * Returns the #GstVaapiDisplayType of the VA display bound to
+ * @display. This is not the type of the @display object.
  *
  * Return value: the #GstVaapiDisplayType
  */
@@ -1291,7 +1312,7 @@ gst_vaapi_display_get_display_type (GstVaapiDisplay * display)
 {
   g_return_val_if_fail (display != NULL, GST_VAAPI_DISPLAY_TYPE_ANY);
 
-  return GST_VAAPI_DISPLAY_GET_PRIVATE (display)->display_type;
+  return GST_VAAPI_DISPLAY_VADISPLAY_TYPE (display);
 }
 
 /**
@@ -2094,3 +2115,26 @@ gst_vaapi_display_get_vendor_string (GstVaapiDisplay * display)
     return NULL;
   return display->priv.vendor_string;
 }
+
+/**
+ * gst_vaapi_display_has_opengl:
+ * @display: a #GstVaapiDisplay
+ *
+ * Returns wether the @display that was created does support OpenGL
+ * context to be attached.
+ *
+ * This function is thread safe.
+ *
+ * Return value: %TRUE if the @display supports OpenGL context, %FALSE
+ *   otherwise
+ */
+gboolean
+gst_vaapi_display_has_opengl (GstVaapiDisplay * display)
+{
+  GstVaapiDisplayClass *klass;
+
+  g_return_val_if_fail (display != NULL, NULL);
+
+  klass = GST_VAAPI_DISPLAY_GET_CLASS (display);
+  return (klass->display_type == GST_VAAPI_DISPLAY_TYPE_GLX);
+}
