@@ -36,6 +36,51 @@ G_BEGIN_DECLS
 #define GST_VAAPI_DISPLAY(obj) \
     ((GstVaapiDisplay *)(obj))
 
+/**
+ * GST_VAAPI_DISPLAY_GET_CLASS_TYPE:
+ * @display: a #GstVaapiDisplay
+ *
+ * Returns the #display class type
+ */
+#define GST_VAAPI_DISPLAY_GET_CLASS_TYPE(display) \
+    gst_vaapi_display_get_class_type (GST_VAAPI_DISPLAY (display))
+
+/**
+ * GST_VAAPI_DISPLAY_VADISPLAY_TYPE:
+ * @display: a #GstVaapiDisplay
+ *
+ * Returns the underlying VADisplay @display type.
+ */
+#define GST_VAAPI_DISPLAY_VADISPLAY_TYPE(display) \
+  gst_vaapi_display_get_display_type (GST_VAAPI_DISPLAY (display))
+
+/**
+ * GST_VAAPI_DISPLAY_VADISPLAY:
+ * @display_: a #GstVaapiDisplay
+ *
+ * Macro that evaluates to the #VADisplay of @display.
+ */
+#define GST_VAAPI_DISPLAY_VADISPLAY(display) \
+  gst_vaapi_display_get_display (GST_VAAPI_DISPLAY (display))
+
+/**
+ * GST_VAAPI_DISPLAY_LOCK:
+ * @display: a #GstVaapiDisplay
+ *
+ * Locks @display
+ */
+#define GST_VAAPI_DISPLAY_LOCK(display) \
+  gst_vaapi_display_lock (GST_VAAPI_DISPLAY (display))
+
+/**
+ * GST_VAAPI_DISPLAY_UNLOCK:
+ * @display: a #GstVaapiDisplay
+ *
+ * Unlocks @display
+ */
+#define GST_VAAPI_DISPLAY_UNLOCK(display) \
+  gst_vaapi_display_unlock (GST_VAAPI_DISPLAY (display))
+
 typedef struct _GstVaapiDisplayInfo             GstVaapiDisplayInfo;
 typedef struct _GstVaapiDisplay                 GstVaapiDisplay;
 
@@ -121,6 +166,9 @@ gst_vaapi_display_sync (GstVaapiDisplay * display);
 void
 gst_vaapi_display_flush (GstVaapiDisplay * display);
 
+GstVaapiDisplayType
+gst_vaapi_display_get_class_type (GstVaapiDisplay * display);
+
 GstVaapiDisplayType
 gst_vaapi_display_get_display_type (GstVaapiDisplay * display);
 
@@ -204,6 +252,9 @@ gst_vaapi_display_set_rotation (GstVaapiDisplay * display,
 const gchar *
 gst_vaapi_display_get_vendor_string (GstVaapiDisplay * display);
 
+gboolean
+gst_vaapi_display_has_opengl (GstVaapiDisplay * display);
+
 G_END_DECLS
 
 #endif /* GST_VAAPI_DISPLAY_H */
