@@ -36,12 +36,22 @@
 #include "gstvaapidisplay_priv.h"
 #include "gstvaapidisplay_drm.h"
 #include "gstvaapidisplay_drm_priv.h"
+#include "gstvaapiwindow_drm.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
 
 static const guint g_display_types = 1U << GST_VAAPI_DISPLAY_TYPE_DRM;
 
+typedef enum
+{
+  DRM_DEVICE_LEGACY = 1,
+  DRM_DEVICE_RENDERNODES,
+} DRMDeviceType;
+
+static DRMDeviceType g_drm_device_type;
+static GMutex g_drm_device_type_lock;
+
 /* Get default device path. Actually, the first match in the DRM subsystem */
 static const gchar *
 get_default_device_path (GstVaapiDisplay * display)
@@ -65,6 +75,17 @@ get_default_device_path (GstVaapiDisplay * display)
       goto end;
 
     udev_enumerate_add_match_subsystem (e, "drm");
+    switch (g_drm_device_type) {
+      case DRM_DEVICE_LEGACY:
+        udev_enumerate_add_match_sysname (e, "card[0-9]*");
+        break;
+      case DRM_DEVICE_RENDERNODES:
+        udev_enumerate_add_match_sysname (e, "renderD[0-9]*");
+        break;
+      default:
+        GST_ERROR ("unknown drm device type (%d)", g_drm_device_type);
+        goto end;
+    }
     udev_enumerate_scan_devices (e);
     udev_list_entry_foreach (l, udev_enumerate_get_list_entry (e)) {
       syspath = udev_list_entry_get_name (l);
@@ -176,7 +197,8 @@ set_device_path_from_fd (GstVaapiDisplay * display, gint drm_device)
     path_length = strlen (path);
     if (str + busid_length >= path + path_length)
       continue;
-    if (strncmp (&str[busid_length], "/drm/card", 9) != 0)
+    if (strncmp (&str[busid_length], "/drm/card", 9) != 0 &&
+        strncmp (&str[busid_length], "/drm/renderD", 12) != 0)
       continue;
 
     device = udev_device_new_from_syspath (udev, path);
@@ -219,14 +241,14 @@ gst_vaapi_display_drm_open_display (GstVaapiDisplay * display,
 {
   GstVaapiDisplayDRMPrivate *const priv =
       GST_VAAPI_DISPLAY_DRM_PRIVATE (display);
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_GET_CACHE (display);
+  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
   const GstVaapiDisplayInfo *info;
 
   if (!set_device_path (display, name))
     return FALSE;
 
   info = gst_vaapi_display_cache_lookup_by_name (cache, priv->device_path,
-      GST_VAAPI_DISPLAY_TYPES (display));
+      g_display_types);
   if (info) {
     priv->drm_device = GPOINTER_TO_INT (info->native_display);
     priv->use_foreign_display = TRUE;
@@ -268,12 +290,12 @@ gst_vaapi_display_drm_get_display_info (GstVaapiDisplay * display,
 {
   GstVaapiDisplayDRMPrivate *const priv =
       GST_VAAPI_DISPLAY_DRM_PRIVATE (display);
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_GET_CACHE (display);
+  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
   const GstVaapiDisplayInfo *cached_info;
 
   /* Return any cached info even if child has its own VA display */
   cached_info = gst_vaapi_display_cache_lookup_by_native_display (cache,
-      GINT_TO_POINTER (priv->drm_device), GST_VAAPI_DISPLAY_TYPES (display));
+      GINT_TO_POINTER (priv->drm_device), g_display_types);
   if (cached_info) {
     *info = *cached_info;
     return TRUE;
@@ -291,6 +313,14 @@ gst_vaapi_display_drm_get_display_info (GstVaapiDisplay * display,
   return TRUE;
 }
 
+static GstVaapiWindow *
+gst_vaapi_display_drm_create_window (GstVaapiDisplay * display, GstVaapiID id,
+    guint width, guint height)
+{
+  return id != GST_VAAPI_ID_INVALID ?
+      NULL : gst_vaapi_window_drm_new (display, width, height);
+}
+
 static void
 gst_vaapi_display_drm_init (GstVaapiDisplay * display)
 {
@@ -310,12 +340,13 @@ gst_vaapi_display_drm_class_init (GstVaapiDisplayDRMClass * klass)
   gst_vaapi_display_class_init (&klass->parent_class);
 
   object_class->size = sizeof (GstVaapiDisplayDRM);
-  dpy_class->display_types = g_display_types;
+  dpy_class->display_type = GST_VAAPI_DISPLAY_TYPE_DRM;
   dpy_class->init = gst_vaapi_display_drm_init;
   dpy_class->bind_display = gst_vaapi_display_drm_bind_display;
   dpy_class->open_display = gst_vaapi_display_drm_open_display;
   dpy_class->close_display = gst_vaapi_display_drm_close_display;
   dpy_class->get_display = gst_vaapi_display_drm_get_display_info;
+  dpy_class->create_window = gst_vaapi_display_drm_create_window;
 }
 
 static inline const GstVaapiDisplayClass *
@@ -348,8 +379,28 @@ gst_vaapi_display_drm_class (void)
 GstVaapiDisplay *
 gst_vaapi_display_drm_new (const gchar * device_path)
 {
-  return gst_vaapi_display_new (gst_vaapi_display_drm_class (),
-      GST_VAAPI_DISPLAY_INIT_FROM_DISPLAY_NAME, (gpointer) device_path);
+  GstVaapiDisplay *display;
+  guint types[2], i, num_types = 0;
+
+  if (device_path)
+    types[num_types++] = 0;
+  else if (g_drm_device_type)
+    types[num_types++] = g_drm_device_type;
+  else {
+    types[num_types++] = DRM_DEVICE_RENDERNODES;
+    types[num_types++] = DRM_DEVICE_LEGACY;
+  }
+
+  g_mutex_lock (&g_drm_device_type_lock);
+  for (i = 0; i < num_types; i++) {
+    g_drm_device_type = types[i];
+    display = gst_vaapi_display_new (gst_vaapi_display_drm_class (),
+        GST_VAAPI_DISPLAY_INIT_FROM_DISPLAY_NAME, (gpointer) device_path);
+    if (display || device_path)
+      break;
+  }
+  g_mutex_unlock (&g_drm_device_type_lock);
+  return display;
 }
 
 /**
