@@ -35,24 +35,30 @@
 #include "gstvaapidisplay_x11_priv.h"
 #include "gstvaapidisplay_glx.h"
 #include "gstvaapidisplay_glx_priv.h"
+#include "gstvaapiwindow_glx.h"
+#include "gstvaapitexture_glx.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
 
 static const guint g_display_types = 1U << GST_VAAPI_DISPLAY_TYPE_GLX;
 
-static gboolean
-gst_vaapi_display_glx_get_display_info (GstVaapiDisplay * display,
-    GstVaapiDisplayInfo * info)
+static GstVaapiWindow *
+gst_vaapi_display_glx_create_window (GstVaapiDisplay * display, GstVaapiID id,
+    guint width, guint height)
 {
-  const GstVaapiDisplayGLXClass *const klass =
-      GST_VAAPI_DISPLAY_GLX_GET_CLASS (display);
+  return id != GST_VAAPI_ID_INVALID ?
+      gst_vaapi_window_glx_new_with_xid (display, id) :
+      gst_vaapi_window_glx_new (display, width, height);
+}
 
-  info->va_display = vaGetDisplayGLX (GST_VAAPI_DISPLAY_XDISPLAY (display));
-  if (!info->va_display)
-    return FALSE;
-  info->display_type = GST_VAAPI_DISPLAY_TYPE_GLX;
-  return klass->parent_get_display (display, info);
+static GstVaapiTexture *
+gst_vaapi_display_glx_create_texture (GstVaapiDisplay * display, GstVaapiID id,
+    guint target, guint format, guint width, guint height)
+{
+  return id != GST_VAAPI_ID_INVALID ?
+      gst_vaapi_texture_glx_new_wrapped (display, id, target, format) :
+      gst_vaapi_texture_glx_new (display, target, format, width, height);
 }
 
 static void
@@ -65,9 +71,9 @@ gst_vaapi_display_glx_class_init (GstVaapiDisplayGLXClass * klass)
   gst_vaapi_display_x11_class_init (&klass->parent_class);
 
   object_class->size = sizeof (GstVaapiDisplayGLX);
-  klass->parent_get_display = dpy_class->get_display;
-  dpy_class->display_types = g_display_types;
-  dpy_class->get_display = gst_vaapi_display_glx_get_display_info;
+  dpy_class->display_type = GST_VAAPI_DISPLAY_TYPE_GLX;
+  dpy_class->create_window = gst_vaapi_display_glx_create_window;
+  dpy_class->create_texture = gst_vaapi_display_glx_create_texture;
 }
 
 static inline const GstVaapiDisplayClass *
