@@ -31,6 +31,7 @@
 #include "gstvaapidisplay_priv.h"
 #include "gstvaapidisplay_wayland.h"
 #include "gstvaapidisplay_wayland_priv.h"
+#include "gstvaapiwindow_wayland.h"
 
 #define DEBUG 1
 #include "gstvaapidebug.h"
@@ -204,14 +205,14 @@ gst_vaapi_display_wayland_open_display (GstVaapiDisplay * display,
 {
   GstVaapiDisplayWaylandPrivate *const priv =
       GST_VAAPI_DISPLAY_WAYLAND_GET_PRIVATE (display);
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_GET_CACHE (display);
+  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
   const GstVaapiDisplayInfo *info;
 
   if (!set_display_name (display, name))
     return FALSE;
 
   info = gst_vaapi_display_cache_lookup_custom (cache, compare_display_name,
-      priv->display_name, GST_VAAPI_DISPLAY_TYPES (display));
+      priv->display_name, g_display_types);
   if (info) {
     priv->wl_display = info->native_display;
     priv->use_foreign_display = TRUE;
@@ -263,12 +264,12 @@ gst_vaapi_display_wayland_get_display_info (GstVaapiDisplay * display,
 {
   GstVaapiDisplayWaylandPrivate *const priv =
       GST_VAAPI_DISPLAY_WAYLAND_GET_PRIVATE (display);
-  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_GET_CACHE (display);
+  GstVaapiDisplayCache *const cache = GST_VAAPI_DISPLAY_CACHE (display);
   const GstVaapiDisplayInfo *cached_info;
 
   /* Return any cached info even if child has its own VA display */
   cached_info = gst_vaapi_display_cache_lookup_by_native_display (cache,
-      priv->wl_display, GST_VAAPI_DISPLAY_TYPES (display));
+      priv->wl_display, g_display_types);
   if (cached_info) {
     *info = *cached_info;
     return TRUE;
@@ -320,6 +321,15 @@ gst_vaapi_display_wayland_get_size_mm (GstVaapiDisplay * display,
     *pheight = priv->phys_height;
 }
 
+static GstVaapiWindow *
+gst_vaapi_display_wayland_create_window (GstVaapiDisplay * display,
+    GstVaapiID id, guint width, guint height)
+{
+  return id != GST_VAAPI_ID_INVALID ?
+      NULL :
+      gst_vaapi_window_wayland_new (display, width, height);
+}
+
 static void
 gst_vaapi_display_wayland_init (GstVaapiDisplay * display)
 {
@@ -339,7 +349,7 @@ gst_vaapi_display_wayland_class_init (GstVaapiDisplayWaylandClass * klass)
   gst_vaapi_display_class_init (&klass->parent_class);
 
   object_class->size = sizeof (GstVaapiDisplayWayland);
-  dpy_class->display_types = g_display_types;
+  dpy_class->display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
   dpy_class->init = gst_vaapi_display_wayland_init;
   dpy_class->bind_display = gst_vaapi_display_wayland_bind_display;
   dpy_class->open_display = gst_vaapi_display_wayland_open_display;
@@ -347,6 +357,7 @@ gst_vaapi_display_wayland_class_init (GstVaapiDisplayWaylandClass * klass)
   dpy_class->get_display = gst_vaapi_display_wayland_get_display_info;
   dpy_class->get_size = gst_vaapi_display_wayland_get_size;
   dpy_class->get_size_mm = gst_vaapi_display_wayland_get_size_mm;
+  dpy_class->create_window = gst_vaapi_display_wayland_create_window;
 }
 
 static inline const GstVaapiDisplayClass *
