@@ -1005,6 +1005,8 @@ ensure_profile_limits (GstVaapiEncoderH264 * encoder)
 
   GST_WARNING ("lowering coding tools to meet target decoder constraints");
 
+  profile = GST_VAAPI_PROFILE_UNKNOWN;
+
   /* Try Main profile coding tools */
   if (encoder->max_profile_idc < 100) {
     encoder->use_dct8x8 = FALSE;
@@ -1018,8 +1020,10 @@ ensure_profile_limits (GstVaapiEncoderH264 * encoder)
     profile = GST_VAAPI_PROFILE_H264_CONSTRAINED_BASELINE;
   }
 
-  encoder->profile = profile;
-  encoder->profile_idc = encoder->max_profile_idc;
+  if (profile) {
+    encoder->profile = profile;
+    encoder->profile_idc = encoder->max_profile_idc;
+  }
   return TRUE;
 }
 
@@ -1633,7 +1637,10 @@ fill_sequence (GstVaapiEncoderH264 * encoder, GstVaapiEncSequence * sequence)
   seq_param->seq_parameter_set_id = encoder->view_idx;
   seq_param->level_idc = encoder->level_idc;
   seq_param->intra_period = GST_VAAPI_ENCODER_KEYFRAME_PERIOD (encoder);
+  seq_param->intra_idr_period = GST_VAAPI_ENCODER_KEYFRAME_PERIOD (encoder);
   seq_param->ip_period = 1 + encoder->num_bframes;
+  seq_param->ip_period = seq_param->intra_period > 1 ?
+      (1 + encoder->num_bframes) : 0;
   seq_param->bits_per_second = encoder->bitrate_bits;
 
   seq_param->max_num_ref_frames = ref_pool->max_ref_frames;
@@ -1697,9 +1704,10 @@ fill_sequence (GstVaapiEncoderH264 * encoder, GstVaapiEncSequence * sequence)
   /* VUI parameters are always set, at least for timing_info (framerate) */
   seq_param->vui_parameters_present_flag = TRUE;
   if (seq_param->vui_parameters_present_flag) {
-    seq_param->vui_fields.bits.aspect_ratio_info_present_flag = FALSE;
+    seq_param->vui_fields.bits.aspect_ratio_info_present_flag = TRUE;
     if (seq_param->vui_fields.bits.aspect_ratio_info_present_flag) {
       const GstVideoInfo *const vip = GST_VAAPI_ENCODER_VIDEO_INFO (encoder);
+      seq_param->aspect_ratio_idc = 0xff;
       seq_param->sar_width = GST_VIDEO_INFO_PAR_N (vip);
       seq_param->sar_height = GST_VIDEO_INFO_PAR_D (vip);
     }
