@@ -52,6 +52,8 @@ typedef gboolean (*GstVaapiWindowRenderFunc) (GstVaapiWindow * window,
 typedef gboolean (*GstVaapiWindowRenderPixmapFunc) (GstVaapiWindow * window,
     GstVaapiPixmap * pixmap, const GstVaapiRectangle * src_rect,
     const GstVaapiRectangle * dst_rect);
+typedef guintptr (*GstVaapiWindowGetVisualIdFunc) (GstVaapiWindow * window);
+typedef guintptr (*GstVaapiWindowGetColormapFunc) (GstVaapiWindow * window);
 
 /**
  * GstVaapiWindow:
@@ -82,6 +84,10 @@ struct _GstVaapiWindow
  * @set_fullscreen: virtual function to change window fullscreen state
  * @resize: virtual function to resize a window
  * @render: virtual function to render a #GstVaapiSurface into a window
+ * @get_visual_id: virtual function to get the desired visual id used to
+ *   create the window
+ * @get_colormap: virtual function to get the desired colormap used to
+ *   create the window, or the currently allocated one
  *
  * Base class for system-dependent windows.
  */
@@ -99,15 +105,13 @@ struct _GstVaapiWindowClass
   GstVaapiWindowResizeFunc resize;
   GstVaapiWindowRenderFunc render;
   GstVaapiWindowRenderPixmapFunc render_pixmap;
+  GstVaapiWindowGetVisualIdFunc get_visual_id;
+  GstVaapiWindowGetColormapFunc get_colormap;
 };
 
 GstVaapiWindow *
-gst_vaapi_window_new (const GstVaapiWindowClass * window_class,
-    GstVaapiDisplay * display, guint width, guint height);
-
-GstVaapiWindow *
-gst_vaapi_window_new_from_native (const GstVaapiWindowClass *
-    window_class, GstVaapiDisplay * display, gpointer native_window);
+gst_vaapi_window_new_internal (const GstVaapiWindowClass * window_class,
+    GstVaapiDisplay * display, guintptr handle, guint width, guint height);
 
 /* Inline reference counting for core libgstvaapi library */
 #ifdef IN_LIBGSTVAAPI_CORE
