@@ -163,7 +163,8 @@ gst_vaapi_window_wayland_sync (GstVaapiWindow * window)
       GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
 
   if (priv->frame) {
-    struct wl_display *const wl_display = GST_VAAPI_OBJECT_WL_DISPLAY (window);
+    struct wl_display *const wl_display =
+        GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
 
     do {
       if (wl_display_dispatch_queue (wl_display, priv->event_queue) < 0)
@@ -347,14 +348,12 @@ vpp_convert (GstVaapiWindow * window,
   GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (window);
   GstVaapiSurface *vpp_surface = NULL;
   GstVaapiFilterStatus status;
-  GstVideoInfo vi;
 
   /* Ensure VA surface pool is created */
   /* XXX: optimize the surface format to use. e.g. YUY2 */
   if (!priv->surface_pool) {
-    gst_video_info_set_format (&vi, priv->surface_format,
-        window->width, window->height);
-    priv->surface_pool = gst_vaapi_surface_pool_new (display, &vi);
+    priv->surface_pool = gst_vaapi_surface_pool_new (display,
+        priv->surface_format, window->width, window->height);
     if (!priv->surface_pool)
       return NULL;
     gst_vaapi_filter_replace (&priv->filter, NULL);
@@ -409,7 +408,8 @@ gst_vaapi_window_wayland_render (GstVaapiWindow * window,
   GstVaapiWindowWaylandPrivate *const priv =
       GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
   GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (window);
-  struct wl_display *const wl_display = GST_VAAPI_OBJECT_WL_DISPLAY (window);
+  struct wl_display *const wl_display =
+      GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
   struct wl_buffer *buffer;
   FrameState *frame;
   guint width, height, va_flags;
@@ -543,7 +543,7 @@ gst_vaapi_window_wayland_new (GstVaapiDisplay * display,
 
   g_return_val_if_fail (GST_VAAPI_IS_DISPLAY_WAYLAND (display), NULL);
 
-  return
-      gst_vaapi_window_new (GST_VAAPI_WINDOW_CLASS
-      (gst_vaapi_window_wayland_class ()), display, width, height);
+  return gst_vaapi_window_new_internal (GST_VAAPI_WINDOW_CLASS
+      (gst_vaapi_window_wayland_class ()), display, GST_VAAPI_ID_INVALID, width,
+      height);
 }
