@@ -115,11 +115,16 @@ gst_vaapi_decoder_state_changed(GstVaapiDecoder *decoder,
 {
     GstVaapiDecode * const decode = GST_VAAPIDECODE(user_data);
     GstVideoDecoder * const vdec = GST_VIDEO_DECODER(decode);
+    GstVaapiPluginBase * const plugin = GST_VAAPI_PLUGIN_BASE(vdec);
 
     g_assert(decode->decoder == decoder);
 
-    gst_vaapidecode_update_src_caps(decode, codec_state);
-    gst_video_decoder_negotiate(vdec);
+    if (gst_vaapidecode_update_src_caps(decode, codec_state)) {
+        if (!gst_video_decoder_negotiate(vdec))
+            return;
+        if (!gst_vaapi_plugin_base_set_caps(plugin, NULL, decode->srcpad_caps))
+            return;
+   }
 }
 
 static inline gboolean
@@ -129,25 +134,6 @@ gst_vaapidecode_update_sink_caps(GstVaapiDecode *decode, GstCaps *caps)
     return TRUE;
 }
 
-#if GST_CHECK_VERSION(1,1,0)
-static void
-gst_vaapidecode_video_info_change_format(GstVideoInfo *info,
-    GstVideoFormat format, guint width, guint height)
-{
-    GstVideoInfo vi = *info;
-
-    gst_video_info_set_format (info, format, width, height);
-
-    info->interlace_mode = vi.interlace_mode;
-    info->flags = vi.flags;
-    info->views = vi.views;
-    info->par_n = vi.par_n;
-    info->par_d = vi.par_d;
-    info->fps_n = vi.fps_n;
-    info->fps_d = vi.fps_d;
-}
-#endif
-
 static gboolean
 gst_vaapidecode_update_src_caps(GstVaapiDecode *decode,
     const GstVideoCodecState *ref_state)
@@ -170,7 +156,7 @@ gst_vaapidecode_update_src_caps(GstVaapiDecode *decode,
     state = gst_video_decoder_set_output_state(vdec, format,
         ref_state->info.width, ref_state->info.height,
         (GstVideoCodecState *)ref_state);
-    if (!state)
+    if (!state || state->info.width == 0 || state->info.height == 0)
         return FALSE;
 
     vi = &state->info;
@@ -188,7 +174,7 @@ gst_vaapidecode_update_src_caps(GstVaapiDecode *decode,
     vis = *vi;
     switch (feature) {
     case GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META:
-        gst_vaapidecode_video_info_change_format(&vis, GST_VIDEO_FORMAT_RGBA,
+        gst_video_info_change_format(&vis, GST_VIDEO_FORMAT_RGBA,
             GST_VIDEO_INFO_WIDTH(vi), GST_VIDEO_INFO_HEIGHT(vi));
         features = gst_caps_features_new(
             GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META, NULL);
@@ -199,7 +185,7 @@ gst_vaapidecode_update_src_caps(GstVaapiDecode *decode,
                format=ENCODED + memory:VASurface caps feature are provided.
                Meanwhile, providing a random format here works but this is
                a terribly wrong thing per se. */
-            gst_vaapidecode_video_info_change_format(&vis, out_format,
+            gst_video_info_change_format(&vis, out_format,
                 GST_VIDEO_INFO_WIDTH(vi), GST_VIDEO_INFO_HEIGHT(vi));
 #if GST_CHECK_VERSION(1,5,0)
             if (feature == GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE)
@@ -275,7 +261,8 @@ gst_vaapidecode_decode_frame(GstVideoDecoder *vdec, GstVideoCodecFrame *frame)
     /* ERRORS */
 error_decode_loop:
     {
-        GST_ERROR("decode loop error %d", decode->decoder_loop_status);
+        if (decode->decoder_loop_status != GST_FLOW_FLUSHING)
+            GST_ERROR("decode loop error %d", decode->decoder_loop_status);
         gst_video_decoder_drop_frame(vdec, frame);
         return decode->decoder_loop_status;
     }
@@ -444,7 +431,6 @@ gst_vaapidecode_decode_loop(GstVaapiDecode *decode)
        data to complete decoding (there no more data to feed in) */
     if (decode->decoder_finish) {
         g_mutex_lock(&decode->decoder_mutex);
-        decode->decoder_loop_status = GST_FLOW_EOS;
         g_cond_signal(&decode->decoder_finish_done);
         g_mutex_unlock(&decode->decoder_mutex);
         return;
@@ -464,6 +450,9 @@ gst_vaapidecode_flush(GstVideoDecoder *vdec)
     GstVaapiDecode * const decode = GST_VAAPIDECODE(vdec);
     GstVaapiDecoderStatus status;
 
+    if (!decode->decoder)
+      return TRUE;
+
     /* If there is something in GstVideoDecoder's output adapter, then
        submit the frame for decoding */
     if (decode->current_frame_size) {
@@ -490,20 +479,26 @@ gst_vaapidecode_finish(GstVideoDecoder *vdec)
     GstVaapiDecode * const decode = GST_VAAPIDECODE(vdec);
     GstFlowReturn ret = GST_FLOW_OK;
 
+    if (!decode->decoder)
+      return GST_FLOW_OK;
+
     if (!gst_vaapidecode_flush(vdec))
         ret = GST_FLOW_OK;
 
     /* Make sure the decode loop function has a chance to return, thus
        possibly unlocking gst_video_decoder_finish_frame() */
-    decode->decoder_finish = TRUE;
     if (decode->decoder_loop_status == GST_FLOW_OK) {
+        decode->decoder_finish = TRUE;
         GST_VIDEO_DECODER_STREAM_UNLOCK(vdec);
         g_mutex_lock(&decode->decoder_mutex);
-        while (decode->decoder_loop_status != GST_FLOW_OK)
+        while (decode->decoder_loop_status == GST_FLOW_OK)
             g_cond_wait(&decode->decoder_finish_done, &decode->decoder_mutex);
         g_mutex_unlock(&decode->decoder_mutex);
         gst_pad_stop_task(GST_VAAPI_PLUGIN_BASE_SRC_PAD(decode));
         GST_VIDEO_DECODER_STREAM_LOCK(vdec);
+        decode->decoder_finish = FALSE;
+        gst_pad_start_task(GST_VAAPI_PLUGIN_BASE_SRC_PAD(decode),
+            (GstTaskFunction)gst_vaapidecode_decode_loop, decode, NULL);
     }
     return ret;
 }
@@ -514,138 +509,37 @@ gst_vaapidecode_decide_allocation(GstVideoDecoder *vdec, GstQuery *query)
 {
     GstVaapiDecode * const decode = GST_VAAPIDECODE(vdec);
     GstCaps *caps = NULL;
-    GstBufferPool *pool;
-    GstStructure *config;
-    GstVideoInfo vi;
-    guint size, min, max;
-    gboolean need_pool, update_pool;
-    gboolean has_video_meta = FALSE;
-    gboolean has_video_alignment = FALSE;
-#if GST_CHECK_VERSION(1,1,0) && USE_GLX
-    gboolean has_texture_upload_meta = FALSE;
-#endif
+    gboolean need_pool;
     GstVideoCodecState *state;
+    GstVaapiCapsFeature feature;
 
     gst_query_parse_allocation(query, &caps, &need_pool);
 
-    if (!caps)
-        goto error_no_caps;
-
-    state = gst_video_decoder_get_output_state(vdec);
-
     decode->has_texture_upload_meta = FALSE;
-    has_video_meta = gst_query_find_allocation_meta(query,
-        GST_VIDEO_META_API_TYPE, NULL);
-
 #if GST_CHECK_VERSION(1,1,0) && USE_GLX
-    has_texture_upload_meta = gst_query_find_allocation_meta(query,
-        GST_VIDEO_GL_TEXTURE_UPLOAD_META_API_TYPE, NULL);
-
     decode->has_texture_upload_meta =
         gst_vaapi_find_preferred_caps_feature(GST_VIDEO_DECODER_SRC_PAD(vdec),
             GST_VIDEO_FORMAT_ENCODED) ==
         GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META;
 #endif
 
+    feature = decode->has_texture_upload_meta ?
+        GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META :
+        GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE;
+
     /* Update src caps if feature is not handled downstream */
+    state = gst_video_decoder_get_output_state(vdec);
     if (!gst_caps_is_always_compatible(caps, state->caps)) {
         if (decode->has_texture_upload_meta)
-            gst_video_info_set_format(&state->info, GST_VIDEO_FORMAT_RGBA,
+            gst_video_info_change_format(&state->info, GST_VIDEO_FORMAT_RGBA,
                 GST_VIDEO_INFO_WIDTH(&state->info),
                 GST_VIDEO_INFO_HEIGHT(&state->info));
         gst_vaapidecode_update_src_caps(decode, state);
     }
     gst_video_codec_state_unref(state);
 
-    gst_video_info_init(&vi);
-    gst_video_info_from_caps(&vi, caps);
-    if (GST_VIDEO_INFO_FORMAT(&vi) == GST_VIDEO_FORMAT_ENCODED)
-        gst_video_info_set_format(&vi, GST_VIDEO_FORMAT_I420,
-            GST_VIDEO_INFO_WIDTH(&vi), GST_VIDEO_INFO_HEIGHT(&vi));
-
-    g_return_val_if_fail(GST_VAAPI_PLUGIN_BASE_DISPLAY(decode) != NULL, FALSE);
-
-    if (gst_query_get_n_allocation_pools(query) > 0) {
-        gst_query_parse_nth_allocation_pool(query, 0, &pool, &size, &min, &max);
-        size = MAX(size, vi.size);
-        update_pool = TRUE;
-
-        /* Check whether downstream element proposed a bufferpool but did
-           not provide a correct propose_allocation() implementation */
-        has_video_alignment = gst_buffer_pool_has_option(pool,
-            GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
-    }
-    else {
-        pool = NULL;
-        size = vi.size;
-        min = max = 0;
-        update_pool = FALSE;
-    }
-
-    if (!pool || !gst_buffer_pool_has_option(pool,
-            GST_BUFFER_POOL_OPTION_VAAPI_VIDEO_META)) {
-        GST_INFO("no pool or doesn't support GstVaapiVideoMeta, "
-            "making new pool");
-        if (pool)
-            gst_object_unref(pool);
-        pool = gst_vaapi_video_buffer_pool_new(
-            GST_VAAPI_PLUGIN_BASE_DISPLAY(decode));
-        if (!pool)
-            goto error_create_pool;
-
-        config = gst_buffer_pool_get_config(pool);
-        gst_buffer_pool_config_set_params(config, caps, size, min, max);
-        gst_buffer_pool_config_add_option(config,
-            GST_BUFFER_POOL_OPTION_VAAPI_VIDEO_META);
-        gst_buffer_pool_set_config(pool, config);
-    }
-
-    if (has_video_meta) {
-        config = gst_buffer_pool_get_config(pool);
-        gst_buffer_pool_config_add_option(config,
-            GST_BUFFER_POOL_OPTION_VIDEO_META);
-#if GST_CHECK_VERSION(1,1,0) && USE_GLX
-        if (has_texture_upload_meta)
-            gst_buffer_pool_config_add_option(config,
-                GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
-#endif
-        gst_buffer_pool_set_config(pool, config);
-    }
-    else if (has_video_alignment) {
-        config = gst_buffer_pool_get_config(pool);
-        gst_buffer_pool_config_add_option(config,
-            GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
-        gst_buffer_pool_set_config(pool, config);
-    }
-
-#if GST_CHECK_VERSION(1,1,0) && USE_GLX
-    if (decode->has_texture_upload_meta && !has_texture_upload_meta) {
-        config = gst_buffer_pool_get_config(pool);
-        gst_buffer_pool_config_add_option(config,
-            GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
-        gst_buffer_pool_set_config(pool, config);
-    }
-#endif
-
-    if (update_pool)
-        gst_query_set_nth_allocation_pool(query, 0, pool, size, min, max);
-    else
-        gst_query_add_allocation_pool(query, pool, size, min, max);
-    if (pool)
-        gst_object_unref(pool);
-    return TRUE;
-
-    /* ERRORS */
-error_no_caps:
-    {
-        GST_ERROR("no caps specified");
-        return FALSE;
-    }
-error_create_pool:
-    {
-        GST_ERROR("failed to create buffer pool");
-        return FALSE;
-    }
+    return gst_vaapi_plugin_base_decide_allocation(GST_VAAPI_PLUGIN_BASE(vdec),
+        query, feature);
 }
 #endif
 
@@ -847,15 +741,17 @@ gst_vaapidecode_set_format(GstVideoDecoder *vdec, GstVideoCodecState *state)
 
     if (!gst_vaapidecode_update_sink_caps(decode, state->caps))
         return FALSE;
-    if (!gst_vaapidecode_update_src_caps(decode, state))
-        return FALSE;
-    if (!gst_video_decoder_negotiate(vdec))
-        return FALSE;
-    if (!gst_vaapi_plugin_base_set_caps(plugin, decode->sinkpad_caps,
-            decode->srcpad_caps))
+    if (!gst_vaapi_plugin_base_set_caps(plugin, decode->sinkpad_caps, NULL))
         return FALSE;
     if (!gst_vaapidecode_reset_full(decode, decode->sinkpad_caps, FALSE))
         return FALSE;
+
+    if (gst_vaapidecode_update_src_caps(decode, state)) {
+        if (!gst_video_decoder_negotiate(vdec))
+            return FALSE;
+        if (!gst_vaapi_plugin_base_set_caps(plugin, NULL, decode->srcpad_caps))
+            return FALSE;
+    }
     return TRUE;
 }
 
@@ -923,7 +819,13 @@ gst_vaapidecode_change_state (GstElement * element, GstStateChange transition)
 
     switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+        g_mutex_lock(&decode->decoder_mutex);
+        decode->decoder_finish = TRUE;
+        g_cond_signal(&decode->decoder_finish_done);
+        g_cond_signal(&decode->decoder_ready);
+        g_mutex_unlock(&decode->decoder_mutex);
         gst_pad_stop_task(GST_VAAPI_PLUGIN_BASE_SRC_PAD(decode));
+        decode->decoder_finish = FALSE;
         break;
     default:
         break;
@@ -1004,6 +906,8 @@ gst_vaapidecode_ensure_allowed_caps(GstVaapiDecode *decode)
         const GstVaapiProfile profile =
             g_array_index(profiles, GstVaapiProfile, i);
         const gchar *media_type_name;
+        const gchar *profile_name;
+        GstStructure *structure;
 
         media_type_name = gst_vaapi_profile_get_media_type_name(profile);
         if (!media_type_name)
@@ -1012,9 +916,16 @@ gst_vaapidecode_ensure_allowed_caps(GstVaapiDecode *decode)
         caps = gst_caps_from_string(media_type_name);
         if (!caps)
             continue;
+        structure = gst_caps_get_structure (caps, 0);
+
+        profile_name = gst_vaapi_profile_get_name(profile);
+        if (profile_name)
+            gst_structure_set(structure, "profile", G_TYPE_STRING,
+                profile_name, NULL);
+
         allowed_caps = gst_caps_merge(allowed_caps, caps);
     }
-    decode->allowed_caps = allowed_caps;
+    decode->allowed_caps = gst_caps_simplify (allowed_caps);
 
     g_array_unref(profiles);
     return TRUE;
