@@ -23,6 +23,8 @@
  */
 
 #include "gst/vaapi/sysdeps.h"
+#include <gst/vaapi/gstvaapisurface_drm.h>
+#include <gst/base/gstpushsrc.h>
 #include "gstvaapipluginbase.h"
 #include "gstvaapipluginutil.h"
 #include "gstvaapivideocontext.h"
@@ -30,6 +32,9 @@
 #if GST_CHECK_VERSION(1,0,0)
 #include "gstvaapivideobufferpool.h"
 #endif
+#if GST_CHECK_VERSION(1,1,0)
+#include <gst/allocators/allocators.h>
+#endif
 
 /* Default debug category is from the subclass */
 #define GST_CAT_DEFAULT (plugin->debug_category)
@@ -127,6 +132,98 @@ default_display_changed (GstVaapiPluginBase * plugin)
 {
 }
 
+static gboolean
+plugin_update_sinkpad_info_from_buffer (GstVaapiPluginBase * plugin,
+    GstBuffer * buf)
+{
+  GstVideoInfo *const vip = &plugin->sinkpad_info;
+  GstVideoMeta *vmeta;
+  guint i;
+
+  vmeta = gst_buffer_get_video_meta (buf);
+  if (!vmeta)
+    return TRUE;
+
+  if (GST_VIDEO_INFO_FORMAT (vip) != vmeta->format ||
+      GST_VIDEO_INFO_WIDTH (vip) != vmeta->width ||
+      GST_VIDEO_INFO_HEIGHT (vip) != vmeta->height ||
+      GST_VIDEO_INFO_N_PLANES (vip) != vmeta->n_planes)
+    return FALSE;
+
+  for (i = 0; i < GST_VIDEO_INFO_N_PLANES (vip); ++i) {
+    GST_VIDEO_INFO_PLANE_OFFSET (vip, i) = vmeta->offset[i];
+    GST_VIDEO_INFO_PLANE_STRIDE (vip, i) = vmeta->stride[i];
+  }
+  GST_VIDEO_INFO_SIZE (vip) = gst_buffer_get_size (buf);
+  return TRUE;
+}
+
+#if GST_CHECK_VERSION(1,1,0)
+static gboolean
+is_dma_buffer (GstBuffer * buf)
+{
+  GstMemory *mem;
+
+  if (gst_buffer_n_memory (buf) < 1)
+    return FALSE;
+
+  mem = gst_buffer_peek_memory (buf, 0);
+  if (!mem || !gst_is_dmabuf_memory (mem))
+    return FALSE;
+  return TRUE;
+}
+
+static gboolean
+plugin_bind_dma_to_vaapi_buffer (GstVaapiPluginBase * plugin,
+    GstBuffer * inbuf, GstBuffer * outbuf)
+{
+  GstVideoInfo *const vip = &plugin->sinkpad_info;
+  GstVaapiVideoMeta *meta;
+  GstVaapiSurface *surface;
+  GstVaapiSurfaceProxy *proxy;
+  gint fd;
+
+  fd = gst_dmabuf_memory_get_fd (gst_buffer_peek_memory (inbuf, 0));
+  if (fd < 0)
+    return FALSE;
+
+  if (!plugin_update_sinkpad_info_from_buffer (plugin, inbuf))
+    goto error_update_sinkpad_info;
+
+  meta = gst_buffer_get_vaapi_video_meta (outbuf);
+  g_return_val_if_fail (meta != NULL, FALSE);
+
+  surface = gst_vaapi_surface_new_with_dma_buf_handle (plugin->display, fd,
+      GST_VIDEO_INFO_SIZE (vip), GST_VIDEO_INFO_FORMAT (vip),
+      GST_VIDEO_INFO_WIDTH (vip), GST_VIDEO_INFO_HEIGHT (vip),
+      vip->offset, vip->stride);
+  if (!surface)
+    goto error_create_surface;
+
+  proxy = gst_vaapi_surface_proxy_new (surface);
+  gst_vaapi_object_unref (surface);
+  if (!proxy)
+    goto error_create_proxy;
+
+  gst_vaapi_surface_proxy_set_destroy_notify (proxy,
+      (GDestroyNotify) gst_buffer_unref, (gpointer) gst_buffer_ref (inbuf));
+  gst_vaapi_video_meta_set_surface_proxy (meta, proxy);
+  gst_vaapi_surface_proxy_unref (proxy);
+  return TRUE;
+
+  /* ERRORS */
+error_update_sinkpad_info:
+  GST_ERROR ("failed to update sink pad video info from video meta");
+  return FALSE;
+error_create_surface:
+  GST_ERROR ("failed to create VA surface from dma_buf handle");
+  return FALSE;
+error_create_proxy:
+  GST_ERROR ("failed to create VA surface proxy from wrapped VA surface");
+  return FALSE;
+}
+#endif
+
 void
 gst_vaapi_plugin_base_class_init (GstVaapiPluginBaseClass * klass)
 {
@@ -164,6 +261,7 @@ void
 gst_vaapi_plugin_base_finalize (GstVaapiPluginBase * plugin)
 {
   gst_vaapi_plugin_base_close (plugin);
+  g_free (plugin->display_name);
   if (plugin->sinkpad)
     gst_object_unref (plugin->sinkpad);
   if (plugin->srcpad)
@@ -206,6 +304,7 @@ gst_vaapi_plugin_base_close (GstVaapiPluginBase * plugin)
     gst_object_unref (plugin->sinkpad_buffer_pool);
     plugin->sinkpad_buffer_pool = NULL;
   }
+  g_clear_object (&plugin->srcpad_buffer_pool);
 #endif
 
   gst_caps_replace (&plugin->srcpad_caps, NULL);
@@ -294,6 +393,42 @@ gst_vaapi_plugin_base_ensure_uploader (GstVaapiPluginBase * plugin)
   return TRUE;
 }
 
+/* Checks whether the supplied pad peer element supports DMABUF sharing */
+/* XXX: this is a workaround to the absence of any proposer way to
+   specify DMABUF memory capsfeatures or bufferpool option to downstream */
+static gboolean
+has_dmabuf_capable_peer (GstVaapiPluginBase * plugin, GstPad * pad)
+{
+  GstPad *other_pad = NULL;
+  GstElement *element = NULL;
+  gchar *element_name = NULL;
+  gboolean is_dmabuf_capable = FALSE;
+  gint v;
+
+  do {
+    other_pad = gst_pad_get_peer (pad);
+    if (!other_pad)
+      break;
+
+    element = gst_pad_get_parent_element (other_pad);
+    if (!element || !GST_IS_PUSH_SRC (element))
+      break;
+
+    element_name = gst_element_get_name (element);
+    if (!element_name || sscanf (element_name, "v4l2src%d", &v) != 1)
+      break;
+
+    v = 0;
+    g_object_get (element, "io-mode", &v, NULL);
+    is_dmabuf_capable = v == 5; /* "dmabuf-import" enum value */
+  } while (0);
+
+  g_free (element_name);
+  g_clear_object (&element);
+  g_clear_object (&other_pad);
+  return is_dmabuf_capable;
+}
+
 /**
  * ensure_sinkpad_buffer_pool:
  * @plugin: a #GstVaapiPluginBase
@@ -389,6 +524,7 @@ gst_vaapi_plugin_base_set_caps (GstVaapiPluginBase * plugin, GstCaps * incaps,
     if (!gst_video_info_from_caps (&plugin->sinkpad_info, incaps))
       return FALSE;
     plugin->sinkpad_caps_changed = TRUE;
+    plugin->sinkpad_caps_is_raw = !gst_caps_has_vaapi_surface (incaps);
   }
 
   if (outcaps && outcaps != plugin->srcpad_caps) {
@@ -398,7 +534,7 @@ gst_vaapi_plugin_base_set_caps (GstVaapiPluginBase * plugin, GstCaps * incaps,
     plugin->srcpad_caps_changed = TRUE;
   }
 
-  if (plugin->uploader && GST_VIDEO_INFO_IS_YUV (&plugin->sinkpad_info)) {
+  if (plugin->uploader && plugin->sinkpad_caps_is_raw) {
     if (!gst_vaapi_uploader_ensure_display (plugin->uploader, plugin->display))
       return FALSE;
     if (!gst_vaapi_uploader_ensure_caps (plugin->uploader,
@@ -437,6 +573,16 @@ gst_vaapi_plugin_base_propose_allocation (GstVaapiPluginBase * plugin,
       return FALSE;
     gst_query_add_allocation_pool (query, plugin->sinkpad_buffer_pool,
         plugin->sinkpad_buffer_size, 0, 0);
+
+    if (has_dmabuf_capable_peer (plugin, plugin->sinkpad)) {
+      GstStructure *const config =
+          gst_buffer_pool_get_config (plugin->sinkpad_buffer_pool);
+
+      gst_buffer_pool_config_add_option (config,
+          GST_BUFFER_POOL_OPTION_DMABUF_MEMORY);
+      if (!gst_buffer_pool_set_config (plugin->sinkpad_buffer_pool, config))
+        goto error_pool_config;
+    }
   }
 
   gst_query_add_allocation_meta (query, GST_VAAPI_VIDEO_META_API_TYPE, NULL);
@@ -449,6 +595,151 @@ error_no_caps:
     GST_ERROR ("no caps specified");
     return FALSE;
   }
+error_pool_config:
+  {
+    GST_ERROR ("failed to reset buffer pool config");
+    return FALSE;
+  }
+}
+#endif
+
+/**
+ * gst_vaapi_plugin_base_decide_allocation:
+ * @plugin: a #GstVaapiPluginBase
+ * @query: the allocation query to parse
+ * @feature: the desired #GstVaapiCapsFeature, or zero to find the
+ *   preferred one
+ *
+ * Decides allocation parameters for the downstream elements.
+ *
+ * Returns: %TRUE if successful, %FALSE otherwise.
+ */
+#if GST_CHECK_VERSION(1,0,0)
+gboolean
+gst_vaapi_plugin_base_decide_allocation (GstVaapiPluginBase * plugin,
+    GstQuery * query, guint feature)
+{
+  GstCaps *caps = NULL;
+  GstBufferPool *pool;
+  GstStructure *config;
+  GstVideoInfo vi;
+  guint size, min, max;
+  gboolean need_pool, update_pool;
+  gboolean has_video_meta = FALSE;
+  gboolean has_video_alignment = FALSE;
+#if GST_CHECK_VERSION(1,1,0) && USE_GLX
+  gboolean has_texture_upload_meta = FALSE;
+#endif
+
+  g_return_val_if_fail (plugin->display != NULL, FALSE);
+
+  gst_query_parse_allocation (query, &caps, &need_pool);
+
+  if (!caps)
+    goto error_no_caps;
+
+  if (!feature)
+    feature =
+        gst_vaapi_find_preferred_caps_feature (plugin->srcpad,
+        GST_VIDEO_FORMAT_ENCODED);
+
+  has_video_meta = gst_query_find_allocation_meta (query,
+      GST_VIDEO_META_API_TYPE, NULL);
+
+#if GST_CHECK_VERSION(1,1,0) && USE_GLX
+  has_texture_upload_meta = gst_query_find_allocation_meta (query,
+      GST_VIDEO_GL_TEXTURE_UPLOAD_META_API_TYPE, NULL);
+#endif
+
+  gst_video_info_init (&vi);
+  gst_video_info_from_caps (&vi, caps);
+  if (GST_VIDEO_INFO_FORMAT (&vi) == GST_VIDEO_FORMAT_ENCODED)
+    gst_video_info_set_format (&vi, GST_VIDEO_FORMAT_I420,
+        GST_VIDEO_INFO_WIDTH (&vi), GST_VIDEO_INFO_HEIGHT (&vi));
+
+  if (gst_query_get_n_allocation_pools (query) > 0) {
+    gst_query_parse_nth_allocation_pool (query, 0, &pool, &size, &min, &max);
+    size = MAX (size, vi.size);
+    update_pool = TRUE;
+
+    /* Check whether downstream element proposed a bufferpool but did
+       not provide a correct propose_allocation() implementation */
+    has_video_alignment = gst_buffer_pool_has_option (pool,
+        GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
+  } else {
+    pool = NULL;
+    size = vi.size;
+    min = max = 0;
+    update_pool = FALSE;
+  }
+
+  /* GstVaapiVideoMeta is mandatory, and this implies VA surface memory */
+  if (!pool || !gst_buffer_pool_has_option (pool,
+          GST_BUFFER_POOL_OPTION_VAAPI_VIDEO_META)) {
+    GST_INFO_OBJECT (plugin, "no pool or doesn't support GstVaapiVideoMeta, "
+        "making new pool");
+    if (pool)
+      gst_object_unref (pool);
+    pool = gst_vaapi_video_buffer_pool_new (plugin->display);
+    if (!pool)
+      goto error_create_pool;
+
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_set_params (config, caps, size, min, max);
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_VAAPI_VIDEO_META);
+    gst_buffer_pool_set_config (pool, config);
+  }
+
+  /* Check whether GstVideoMeta, or GstVideoAlignment, is needed (raw video) */
+  if (has_video_meta) {
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_VIDEO_META);
+#if GST_CHECK_VERSION(1,1,0) && USE_GLX
+    if (has_texture_upload_meta)
+      gst_buffer_pool_config_add_option (config,
+          GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
+#endif
+    gst_buffer_pool_set_config (pool, config);
+  } else if (has_video_alignment) {
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_VIDEO_ALIGNMENT);
+    gst_buffer_pool_set_config (pool, config);
+  }
+
+  /* GstVideoGLTextureUploadMeta (OpenGL) */
+#if GST_CHECK_VERSION(1,1,0) && USE_GLX
+  if (feature == GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META
+      && !has_texture_upload_meta) {
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_VIDEO_GL_TEXTURE_UPLOAD_META);
+    gst_buffer_pool_set_config (pool, config);
+  }
+#endif
+
+  if (update_pool)
+    gst_query_set_nth_allocation_pool (query, 0, pool, size, min, max);
+  else
+    gst_query_add_allocation_pool (query, pool, size, min, max);
+
+  g_clear_object (&plugin->srcpad_buffer_pool);
+  plugin->srcpad_buffer_pool = pool;
+  return TRUE;
+
+  /* ERRORS */
+error_no_caps:
+  {
+    GST_ERROR_OBJECT (plugin, "no caps specified");
+    return FALSE;
+  }
+error_create_pool:
+  {
+    GST_ERROR_OBJECT (plugin, "failed to create buffer pool");
+    return FALSE;
+  }
 }
 #endif
 
@@ -477,7 +768,7 @@ gst_vaapi_plugin_base_allocate_input_buffer (GstVaapiPluginBase * plugin,
     plugin->sinkpad_caps_changed = TRUE;
   }
 
-  if (!GST_VIDEO_INFO_IS_YUV (&plugin->sinkpad_info))
+  if (!plugin->sinkpad_caps_is_raw)
     return GST_FLOW_OK;
 
   if (!gst_vaapi_uploader_ensure_display (plugin->uploader, plugin->display))
@@ -529,7 +820,7 @@ gst_vaapi_plugin_base_get_input_buffer (GstVaapiPluginBase * plugin,
     return GST_FLOW_OK;
   }
 
-  if (!GST_VIDEO_INFO_IS_YUV (&plugin->sinkpad_info))
+  if (!plugin->sinkpad_caps_is_raw)
     goto error_invalid_buffer;
 
   if (!plugin->sinkpad_buffer_pool)
@@ -543,6 +834,14 @@ gst_vaapi_plugin_base_get_input_buffer (GstVaapiPluginBase * plugin,
           &outbuf, NULL) != GST_FLOW_OK)
     goto error_create_buffer;
 
+#if GST_CHECK_VERSION(1,1,0)
+  if (is_dma_buffer (inbuf)) {
+    if (!plugin_bind_dma_to_vaapi_buffer (plugin, inbuf, outbuf))
+      goto error_bind_dma_buffer;
+    goto done;
+  }
+#endif
+
   if (!gst_video_frame_map (&src_frame, &plugin->sinkpad_info, inbuf,
           GST_MAP_READ))
     goto error_map_src_buffer;
@@ -557,7 +856,9 @@ gst_vaapi_plugin_base_get_input_buffer (GstVaapiPluginBase * plugin,
   if (!success)
     goto error_copy_buffer;
 
-  gst_buffer_copy_into (outbuf, inbuf, GST_BUFFER_COPY_TIMESTAMPS, 0, -1);
+done:
+  gst_buffer_copy_into (outbuf, inbuf, GST_BUFFER_COPY_FLAGS |
+      GST_BUFFER_COPY_TIMESTAMPS, 0, -1);
   *outbuf_ptr = outbuf;
   return GST_FLOW_OK;
 
@@ -586,7 +887,7 @@ error_map_src_buffer:
 #else
   if (meta)
     outbuf = gst_buffer_ref (inbuf);
-  else if (GST_VIDEO_INFO_IS_YUV (&plugin->sinkpad_info)) {
+  else if (plugin->sinkpad_caps_is_raw) {
     outbuf = gst_vaapi_uploader_get_buffer (plugin->uploader);
     if (!outbuf)
       goto error_create_buffer;
@@ -595,7 +896,7 @@ error_map_src_buffer:
   } else
     goto error_invalid_buffer;
 
-  if (GST_VIDEO_INFO_IS_YUV (&plugin->sinkpad_info) &&
+  if (plugin->sinkpad_caps_is_raw &&
       !gst_vaapi_uploader_process (plugin->uploader, inbuf, outbuf))
     goto error_copy_buffer;
 
@@ -614,6 +915,12 @@ error_create_buffer:
     GST_ERROR ("failed to create buffer");
     return GST_FLOW_ERROR;
   }
+error_bind_dma_buffer:
+  {
+    GST_ERROR ("failed to bind dma_buf to VA surface buffer");
+    gst_buffer_unref (outbuf);
+    return GST_FLOW_ERROR;
+  }
 error_copy_buffer:
   {
     GST_WARNING ("failed to upload buffer to VA surface");
