@@ -274,9 +274,6 @@ gst_vaapi_reply_to_query (GstQuery * query, GstVaapiDisplay * display)
         }
 #endif
 #if USE_X11
-#if USE_GLX
-        case GST_VAAPI_DISPLAY_TYPE_GLX:
-#endif
         case GST_VAAPI_DISPLAY_TYPE_X11:{
           GstVaapiDisplayX11 *const xvadpy = GST_VAAPI_DISPLAY_X11 (display);
           Display *const x11dpy = gst_vaapi_display_x11_get_display (xvadpy);
@@ -347,17 +344,14 @@ gst_vaapi_apply_composition (GstVaapiSurface * surface, GstBuffer * buffer)
 #if GST_CHECK_VERSION(1,0,0)
   GstVideoOverlayCompositionMeta *const cmeta =
       gst_buffer_get_video_overlay_composition_meta (buffer);
-  GstVideoOverlayComposition *composition;
+  GstVideoOverlayComposition *composition = NULL;
 
-  if (!cmeta)
-    return TRUE;
-  composition = cmeta->overlay;
+  if (cmeta)
+    composition = cmeta->overlay;
 #else
   GstVideoOverlayComposition *const composition =
       gst_video_buffer_get_overlay_composition (buffer);
 #endif
-  if (!composition)
-    return TRUE;
   return gst_vaapi_surface_set_subpictures_from_composition (surface,
       composition, TRUE);
 }
@@ -456,6 +450,7 @@ gst_vaapi_video_format_new_template_caps_from_list (GArray * formats)
 
   gst_caps_set_value (caps, "format", &v_formats);
   set_video_template_caps (caps);
+  g_value_unset (&v_formats);
 #else
   GstCaps *caps, *tmp_caps;
   guint i;
@@ -581,6 +576,30 @@ cleanup:
   return feature;
 }
 
+const gchar *
+gst_vaapi_caps_feature_to_string (GstVaapiCapsFeature feature)
+{
+  const gchar *str;
+
+  switch (feature) {
+#if GST_CHECK_VERSION(1,1,0)
+    case GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY:
+      str = GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY;
+      break;
+    case GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META:
+      str = GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META;
+      break;
+    case GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE:
+      str = GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE;
+      break;
+#endif
+    default:
+      str = NULL;
+      break;
+  }
+  return str;
+}
+
 gboolean
 gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip)
 {
@@ -612,3 +631,64 @@ gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip)
 #endif
   return TRUE;
 }
+
+/* Checks whether the supplied caps contain VA surfaces */
+gboolean
+gst_caps_has_vaapi_surface (GstCaps * caps)
+{
+  gboolean found_caps = FALSE;
+  guint i, num_structures;
+
+  g_return_val_if_fail (caps != NULL, FALSE);
+
+  num_structures = gst_caps_get_size (caps);
+  if (num_structures < 1)
+    return FALSE;
+
+#if GST_CHECK_VERSION(1,1,0)
+  for (i = 0; i < num_structures && !found_caps; i++) {
+    GstCapsFeatures *const features = gst_caps_get_features (caps, i);
+
+#if GST_CHECK_VERSION(1,3,0)
+    /* Skip ANY features, we need an exact match for correct evaluation */
+    if (gst_caps_features_is_any (features))
+      continue;
+#endif
+
+    found_caps = gst_caps_features_contains (features,
+        GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
+  }
+#else
+  for (i = 0; i < num_structures && !found_caps; i++) {
+    GstStructure *const structure = gst_caps_get_structure (caps, i);
+    GstCaps *test_caps;
+    GstVideoInfo vi;
+
+    test_caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
+    if (!test_caps)
+      continue;
+
+    found_caps = gst_video_info_from_caps (&vi, test_caps) &&
+        GST_VIDEO_INFO_FORMAT (&vi) == GST_VIDEO_FORMAT_ENCODED;
+    gst_caps_unref (test_caps);
+  }
+#endif
+  return found_caps;
+}
+
+void
+gst_video_info_change_format (GstVideoInfo * vip, GstVideoFormat format,
+    guint width, guint height)
+{
+  GstVideoInfo vi = *vip;
+
+  gst_video_info_set_format (vip, format, width, height);
+
+  vip->interlace_mode = vi.interlace_mode;
+  vip->flags = vi.flags;
+  vip->views = vi.views;
+  vip->par_n = vi.par_n;
+  vip->par_d = vi.par_d;
+  vip->fps_n = vi.fps_n;
+  vip->fps_d = vi.fps_d;
+}
